@@ -95,19 +95,23 @@ static void gc_free(MVMThreadContext *tc, MVMObject *obj) {
     tc->instance->all_scs[sc->body->sc_idx] = NULL;
     uv_mutex_unlock(&tc->instance->mutex_sc_weakhash);
 
-    /* Free manually managed STable list memory. */
-    MVM_checked_free_null(sc->body->root_stables);
+    /* Free manually managed object and STable root list memory. */
+    MVM_free(sc->body->root_objects);
+    MVM_free(sc->body->root_stables);
 
     /* If we have a serialization reader, clean that up too. */
     if (sc->body->sr) {
         if (sc->body->sr->data_needs_free)
-            MVM_checked_free_null(sc->body->sr->data);
-        MVM_checked_free_null(sc->body->sr->contexts);
-        MVM_checked_free_null(sc->body->sr);
+            MVM_free(sc->body->sr->data);
+        MVM_free(sc->body->sr->root.dependent_scs);
+        MVM_free(sc->body->sr->contexts);
+        MVM_free(sc->body->sr->wl_objects.indexes);
+        MVM_free(sc->body->sr->wl_stables.indexes);
+        MVM_free(sc->body->sr);
     }
 
     /* Free body. */
-    MVM_checked_free_null(sc->body);
+    MVM_free(sc->body);
 }
 
 static const MVMStorageSpec storage_spec = {
