@@ -40,7 +40,9 @@ MVMObject * MVM_sc_create(MVMThreadContext *tc, MVMString *handle) {
                 scb->sc = sc;
                 sc->body = scb;
                 MVM_ASSIGN_REF(tc, &(sc->common.header), scb->handle, handle);
+                MVM_gc_allocate_gen2_default_set(tc);
                 MVM_repr_init(tc, (MVMObject *)sc);
+                MVM_gc_allocate_gen2_default_clear(tc);
             }
             uv_mutex_unlock(&tc->instance->mutex_sc_weakhash);
         });
@@ -102,6 +104,14 @@ MVMint64 MVM_sc_find_object_idx(MVMThreadContext *tc, MVMSerializationContext *s
         "Object does not exist in serialization context");
 }
 
+/* Calls MVM_sc_find_object_idx, but first checks if the sc is actually an SCRef. */
+MVMint64 MVM_sc_find_object_idx_jit(MVMThreadContext *tc, MVMObject *sc, MVMObject *obj) {
+    if (REPR(sc)->ID != MVM_REPR_ID_SCRef)
+        MVM_exception_throw_adhoc(tc,
+            "Must provide an SCRef operand to scgetobjidx");
+    return MVM_sc_find_object_idx(tc, (MVMSerializationContext *)sc, obj);
+}
+
 /* Given an SC, looks up the index of an STable that is in its root set. */
 MVMint64 MVM_sc_find_stable_idx(MVMThreadContext *tc, MVMSerializationContext *sc, MVMSTable *st) {
     MVMuint64 i;
@@ -126,11 +136,18 @@ MVMint64 MVM_sc_find_code_idx(MVMThreadContext *tc, MVMSerializationContext *sc,
         if (test == obj)
             return i;
     }
-    MVM_exception_throw_adhoc(tc,
-        "Code ref '%s' does not exist in serialization context",
-        REPR(obj)->ID == MVM_REPR_ID_MVMCode
-            ? MVM_string_utf8_encode_C_string(tc, ((MVMCode *)obj)->body.name)
-            : "<NOT A CODE OBJECT>");
+
+    if (REPR(obj)->ID == MVM_REPR_ID_MVMCode) {
+        char *c_name = MVM_string_utf8_encode_C_string(tc, ((MVMCode *)obj)->body.name);
+        char *waste[] = { c_name, NULL };
+        MVM_exception_throw_adhoc_free(tc, waste,
+            "Code ref '%s' does not exist in serialization context",
+                c_name);
+    }
+    else {
+        MVM_exception_throw_adhoc(tc,
+            "Code ref '<NOT A CODE OBJECT>' does not exist in serialization context");
+    }
 }
 
 /* Given a compilation unit and dependency index, returns that SC. */
@@ -163,10 +180,13 @@ MVMObject * MVM_sc_get_object(MVMThreadContext *tc, MVMSerializationContext *sc,
         return roots[idx] && !sc_working(sc)
             ? roots[idx]
             : MVM_serialization_demand_object(tc, sc, idx);
-    else
-        MVM_exception_throw_adhoc(tc,
+    else {
+        char *c_description = MVM_string_utf8_encode_C_string(tc, sc->body->description);
+        char *waste[] = { c_description, NULL };
+        MVM_exception_throw_adhoc_free(tc, waste,
             "Probable version skew in pre-compiled '%s' (cause: no object at index %"PRId64")",
-            MVM_string_utf8_encode_C_string(tc, sc->body->description), idx);
+            c_description, idx);
+    }
 }
 
 MVMObject * MVM_sc_get_sc_object(MVMThreadContext *tc, MVMCompUnit *cu,
@@ -224,9 +244,11 @@ MVMSTable * MVM_sc_get_stable(MVMThreadContext *tc, MVMSerializationContext *sc,
         return got && !sc_working(sc) ? got : MVM_serialization_demand_stable(tc, sc, idx);
     }
     else {
-        MVM_exception_throw_adhoc(tc,
+        char *c_description = MVM_string_utf8_encode_C_string(tc, sc->body->description);
+        char *waste[] = { c_description, NULL };
+        MVM_exception_throw_adhoc_free(tc, waste,
             "Probable version skew in pre-compiled '%s' (cause: no STable at index %"PRId64")",
-            MVM_string_utf8_encode_C_string(tc, sc->body->description), idx);
+            c_description, idx);
     }
 }
 
@@ -288,9 +310,11 @@ MVMObject * MVM_sc_get_code(MVMThreadContext *tc, MVMSerializationContext *sc, M
             : found;
     }
     else {
-        MVM_exception_throw_adhoc(tc,
+        char *c_description = MVM_string_utf8_encode_C_string(tc, sc->body->description);
+        char *waste[] = { c_description, NULL };
+        MVM_exception_throw_adhoc_free(tc, waste,
             "Probable version skew in pre-compiled '%s' (cause: no code ref at index %"PRId64")",
-            MVM_string_utf8_encode_C_string(tc, sc->body->description), idx);
+            c_description, idx);
     }
 }
 
