@@ -1,7 +1,6 @@
 #include "moar.h"
-#include <stdarg.h>
 
-#if _MSC_VER
+#ifdef _MSC_VER
 #define snprintf _snprintf
 #define vsnprintf _vsnprintf
 #endif
@@ -300,6 +299,13 @@ char * MVM_exception_backtrace_line(MVMThreadContext *tc, MVMFrame *cur_frame, M
             cur_frame->static_info->body.cu->body.strings[string_heap_index])
         : NULL;
 
+    char *filename_c = filename
+	? MVM_string_utf8_encode_C_string(tc, filename)
+	: "<ephemeral file>";
+    char *name_c = name
+	? MVM_string_utf8_encode_C_string(tc, name)
+	: "<anonymous frame>";
+
     /* We may be mid-instruction if exception was thrown at an unfortunate
      * point; try to cope with that. */
     if (instr == MVM_BC_ILLEGAL_OFFSET && offset >= 2)
@@ -309,10 +315,14 @@ char * MVM_exception_backtrace_line(MVMThreadContext *tc, MVMFrame *cur_frame, M
         not_top ? "from" : "  at",
         tmp1 ? tmp1 : "<unknown>",
         line_number,
-        filename ? MVM_string_utf8_encode_C_string(tc, filename) : "<ephemeral file>",
-        name ? MVM_string_utf8_encode_C_string(tc, name) : "<anonymous frame>",
+        filename_c,
+        name_c,
         instr
     );
+    if (filename)
+	MVM_free(filename_c);
+    if (name)
+	MVM_free(name_c);
 
     if (tmp1)
         MVM_free(tmp1);
@@ -456,6 +466,8 @@ static void panic_unhandled_cat(MVMThreadContext *tc, MVMuint32 cat) {
 
 /* Panic over an unhandled exception object. */
 static void panic_unhandled_ex(MVMThreadContext *tc, MVMException *ex) {
+    char *backtrace;
+
     /* If it's a control exception, try promoting it to a catch one; use
      * the category name. */
     if (ex->body.category != MVM_EX_CAT_CATCH)
@@ -466,8 +478,9 @@ static void panic_unhandled_ex(MVMThreadContext *tc, MVMException *ex) {
         panic_unhandled_cat(tc, ex->body.category);
 
     /* Otherwise, dump message and a backtrace. */
-    fprintf(stderr, "Unhandled exception: %s\n",
-        MVM_string_utf8_encode_C_string(tc, ex->body.message));
+    backtrace = MVM_string_utf8_encode_C_string(tc, ex->body.message);
+    fprintf(stderr, "Unhandled exception: %s\n", backtrace);
+    MVM_free(backtrace);
     MVM_dump_backtrace(tc);
     if (crash_on_error)
         abort();
@@ -649,6 +662,7 @@ MVMObject * MVM_exception_newlexotic_from_jit(MVMThreadContext *tc, MVMint32 lab
 /* Unwinds to a lexotic captured handler. */
 void MVM_exception_gotolexotic(MVMThreadContext *tc, MVMint32 handler_idx, MVMStaticFrame *sf) {
     MVMFrame *f, *search;
+    f = NULL;
     search = tc->cur_frame;
     while (search) {
         f = search;
@@ -717,13 +731,30 @@ MVM_NO_RETURN
 void MVM_exception_throw_adhoc(MVMThreadContext *tc, const char *messageFormat, ...) {
     va_list args;
     va_start(args, messageFormat);
-    MVM_exception_throw_adhoc_va(tc, messageFormat, args);
+    MVM_exception_throw_adhoc_free_va(tc, NULL, messageFormat, args);
     va_end(args);
 }
 
 /* Throws an ad-hoc (untyped) exception. */
 MVM_NO_RETURN
 void MVM_exception_throw_adhoc_va(MVMThreadContext *tc, const char *messageFormat, va_list args) {
+    MVM_exception_throw_adhoc_free_va(tc, NULL, messageFormat, args);
+}
+
+/* Throws an ad-hoc (untyped) exception, taking a NULL-terminated array of
+ * char pointers to deallocate after message construction. */
+MVM_NO_RETURN
+void MVM_exception_throw_adhoc_free(MVMThreadContext *tc, char **waste, const char *messageFormat, ...) {
+    va_list args;
+    va_start(args, messageFormat);
+    MVM_exception_throw_adhoc_free_va(tc, waste, messageFormat, args);
+    va_end(args);
+}
+
+/* Throws an ad-hoc (untyped) exception, taking a NULL-terminated array of
+ * char pointers to deallocate after message construction. */
+MVM_NO_RETURN
+void MVM_exception_throw_adhoc_free_va(MVMThreadContext *tc, char **waste, const char *messageFormat, va_list args) {
     LocatedHandler lh;
 
     /* Create and set up an exception object. */
@@ -733,6 +764,13 @@ void MVM_exception_throw_adhoc_va(MVMThreadContext *tc, const char *messageForma
         int        bytes     = vsnprintf(c_message, 1024, messageFormat, args);
         MVMString *message   = MVM_string_utf8_decode(tc, tc->instance->VMString, c_message, bytes);
         MVM_free(c_message);
+
+        /* Clean up after ourselves to avoid leaking C strings. */
+        if (waste) {
+            while(*waste)
+                MVM_free(*waste++);
+        }
+
         MVM_ASSIGN_REF(tc, &(ex->common.header), ex->body.message, message);
         if (tc->cur_frame) {
             ex->body.origin = MVM_frame_inc_ref(tc, tc->cur_frame);
