@@ -80,7 +80,7 @@ MVMint16 MVM_nativecall_get_arg_type(MVMThreadContext *tc, MVMObject *info, MVMi
     else if (strcmp(ctypename, "cppstruct") == 0)
         result = MVM_NATIVECALL_ARG_CPPSTRUCT;
     else if (strcmp(ctypename, "cpointer") == 0)
-        result = MVM_NATIVECALL_ARG_CPOINTER;
+        result = MVM_NATIVECALL_ARG_CPOINTER | get_rw_flag(tc, info);
     else if (strcmp(ctypename, "carray") == 0)
         result = MVM_NATIVECALL_ARG_CARRAY;
     else if (strcmp(ctypename, "cunion") == 0)
@@ -271,7 +271,7 @@ char * MVM_nativecall_unmarshal_string(MVMThreadContext *tc, MVMObject *value, M
                 str = MVM_string_ascii_encode_any(tc, value_str);
                 break;
             case MVM_NATIVECALL_ARG_UTF16STR:
-                str = MVM_string_utf16_encode(tc, value_str);
+                str = MVM_string_utf16_encode(tc, value_str, 0);
                 break;
             default:
                 str = MVM_string_utf8_encode_C_string(tc, value_str);
@@ -373,8 +373,8 @@ static const char *dlerror(void)
 /* Builds up a native call site out of the supplied arguments. */
 void MVM_nativecall_build(MVMThreadContext *tc, MVMObject *site, MVMString *lib,
         MVMString *sym, MVMString *conv, MVMObject *arg_info, MVMObject *ret_info) {
-    char *lib_name = MVM_string_utf8_encode_C_string(tc, lib);
-    char *sym_name = MVM_string_utf8_encode_C_string(tc, sym);
+    char *lib_name = MVM_string_utf8_c8_encode_C_string(tc, lib);
+    char *sym_name = MVM_string_utf8_c8_encode_C_string(tc, sym);
     MVMint16 i;
 
     /* Initialize the object; grab native call part of its body. */
@@ -574,8 +574,8 @@ static MVMObject * nativecall_cast(MVMThreadContext *tc, MVMObject *target_spec,
 }
 
 MVMObject * MVM_nativecall_global(MVMThreadContext *tc, MVMString *lib, MVMString *sym, MVMObject *target_spec, MVMObject *target_type) {
-    char *lib_name = MVM_string_utf8_encode_C_string(tc, lib);
-    char *sym_name = MVM_string_utf8_encode_C_string(tc, sym);
+    char *lib_name = MVM_string_utf8_c8_encode_C_string(tc, lib);
+    char *sym_name = MVM_string_utf8_c8_encode_C_string(tc, sym);
     DLLib *lib_handle;
     void *entry_point;
     MVMObject *ret = NULL;
