@@ -55,9 +55,20 @@ void MVM_tc_destroy(MVMThreadContext *tc) {
     MVM_gc_gen2_destroy(tc->instance, tc->gen2);
 
     /* Free the thread-specific storage */
-    MVM_checked_free_null(tc->gc_work);
-    MVM_checked_free_null(tc->temproots);
-    MVM_checked_free_null(tc->gen2roots);
+    MVM_free(tc->gc_work);
+    MVM_free(tc->temproots);
+    MVM_free(tc->gen2roots);
+
+    /* Free any memory allocated for NFAs and multi-dim indices. */
+    MVM_free(tc->nfa_done);
+    MVM_free(tc->nfa_curst);
+    MVM_free(tc->nfa_nextst);
+    MVM_free(tc->nfa_fates);
+    MVM_free(tc->nfa_longlit);
+    MVM_free(tc->multi_dim_indices);
+
+    /* Free per-thread lexotic cache. */
+    MVM_free(tc->lexotic_cache);
 
     /* Destroy the libuv event loop */
     uv_loop_delete(tc->loop);
