@@ -282,14 +282,17 @@ static void bind_stdio_handle(MVMThreadContext *tc, MVMOSHandle *h, uv_stdio_con
 static void gc_free(MVMThreadContext *tc, MVMObject *h, void *d) {
     MVMIOSyncStreamData *data = (MVMIOSyncStreamData *)d;
     if (data) {
-        if (data->handle && not_std_handle(tc, h)) {
+        if (data->handle) {
             uv_close((uv_handle_t *)data->handle, NULL);
+            uv_run(tc->loop, UV_RUN_DEFAULT);
+            MVM_free(data->handle);
             data->handle = NULL;
         }
         if (data->ds) {
             MVM_string_decodestream_destory(tc, data->ds);
             data->ds = NULL;
         }
+        MVM_string_decode_stream_sep_destroy(tc, &(data->sep_spec));
         MVM_free(data);
     }
 }
