@@ -114,6 +114,11 @@ MVMJitCode * MVM_jit_compile_graph(MVMThreadContext *tc, MVMJitGraph *jg) {
 
 void MVM_jit_destroy_code(MVMThreadContext *tc, MVMJitCode *code) {
     MVM_platform_free_pages(code->func_ptr, code->size);
+    MVM_free(code->labels);
+    MVM_free(code->bb_labels);
+    MVM_free(code->deopts);
+    MVM_free(code->handlers);
+    MVM_free(code->inlines);
     MVM_free(code);
 }
 
