@@ -102,6 +102,10 @@ struct MVMSpeshGraph {
      * allocate fewer of them across our spesh alloc blocks.
      */
     MVMOpInfo *phi_infos;
+
+    /* If this graph was formed from a spesh candidate rather than an
+     * original static frame, the candidate will be stored here. */
+    MVMSpeshCandidate *cand;
 };
 
 /* The default allocation chunk size for memory blocks used to store spesh
@@ -173,6 +177,10 @@ struct MVMSpeshBB {
      * dominance. */
     MVMint32 rpo_idx;
 
+    /* We cache the instruction pointer of the very first instruction so that
+     * we can output a line number for every BB */
+    MVMuint32 initial_pc;
+
     /* Is this block an inlining of another one? */
     MVMint32 inlined;
 };
@@ -255,4 +263,4 @@ MVMSpeshBB * MVM_spesh_graph_linear_prev(MVMThreadContext *tc, MVMSpeshGraph *g,
 void MVM_spesh_graph_mark(MVMThreadContext *tc, MVMSpeshGraph *g, MVMGCWorklist *worklist);
 void MVM_spesh_graph_destroy(MVMThreadContext *tc, MVMSpeshGraph *g);
 MVM_PUBLIC void * MVM_spesh_alloc(MVMThreadContext *tc, MVMSpeshGraph *g, size_t bytes);
-MVMOpInfo *get_phi(MVMThreadContext *tc, MVMSpeshGraph *g, MVMint32 nrargs);
+MVMOpInfo *get_phi(MVMThreadContext *tc, MVMSpeshGraph *g, MVMuint32 nrargs);
